<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is released under commercial license by Lamia Oy.
 *
 * @copyright Copyright (c) 2017 Lamia Oy (https://lamia.fi)
 * @author    Szymon Nosal <simon@lamia.fi>
 */

class Verifone_Data
{

    const BASKET_LIMIT = 48;

    public static function collectData($order, $items = null, $paymentMethod = null, $savePaymentMethod = null)
    {
        $data = self::collectOrderData($order, $paymentMethod, $savePaymentMethod);

        $customer = self::collectCustomerData($order);

        $data['customer'] = $customer;
        $data['products'] = self::_collectProductsData($order, $items, $paymentMethod, $savePaymentMethod);
        $data['address'] = self::_collectAddressData($order);
        $data['shippingAddress'] = self::_collectShippingAddressData($order);

        return $data;
    }

    /** CUSTOMER */
    /**
     * @param $order
     * @return array|null
     */
    public static function collectCustomerData($order = null)
    {
        if ($order === null) {

            $customer = Verifone_Storage::getInstance()->getCustomerModel();

            $customerData = [
                'firstname' => $customer->getFirstname(),
                'lastname' => $customer->getLastname(),
                'phone' => $customer->getTelephone(),
                'email' => $customer->getEmail()
            ];
            $customerId = $customer->isLogged();

        } else {
            $customerData = [
                'firstname' => $order['firstname'],
                'lastname' => $order['lastname'],
                'phone' => $order['telephone'],
                'email' => $order['email']
            ];
            $customerId = $order['customer_id'];
        }

        // set external customer id if available
        $config = Verifone_Config::getInstance();
        if (!empty($config->getExternalCustomerIdField())) {
            $externalData = '';
            if (!empty($externalData)) {
                $customerData['external_id'] = $externalData;
            }
        }

        return $customerData;
    }

    protected static function _collectAddressData($order)
    {
        $addressData = [
            'address_1' => $order['payment_address_1'],
            'address_2' => $order['payment_address_2'],
            'city' => $order['payment_city'],
            'postcode' => $order['payment_postcode'],
            'country' => $order['payment_iso_code_2'],
            'firstname' => $order['payment_firstname'],
            'lastname' => $order['payment_lastname'],
            'telephone' => $order['telephone'],
            'email' => $order['email']
        ];

        $addressData['country_code'] = Verifone_System::convertCountryCode2Numeric($addressData['country']);

        return $addressData;

    }

    protected static function _collectShippingAddressData($order)
    {

        if(
            empty($order['shipping_address_1']) ||
            empty($order['shipping_city']) ||
            empty($order['shipping_postcode']) ||
            empty($order['shipping_iso_code_2']) ||
            empty($order['shipping_firstname']) ||
            empty($order['shipping_lastname'])

        ) {
            return null;
        }

        $addressData = [
            'address_1' => $order['shipping_address_1'],
            'address_2' => $order['shipping_address_2'],
            'city' => $order['shipping_city'],
            'postcode' => $order['shipping_postcode'],
            'country' => $order['shipping_iso_code_2'],
            'firstname' => $order['shipping_firstname'],
            'lastname' => $order['shipping_lastname'],
            'telephone' => $order['telephone'],
            'email' => $order['email']
        ];

        $addressData['country_code'] = Verifone_System::convertCountryCode2Numeric($addressData['country']);

        return $addressData;
    }

    /** ORDER */

    protected static $_paymentMethod = null;
    protected static $_savePaymentMethod = null;

    /**
     * @param $order
     * @param null $paymentMethod
     * @param null $savePaymentMethod
     * @return array
     */
    public static function collectOrderData($order, $paymentMethod = null, $savePaymentMethod = null)
    {
        $config = Verifone_Config::getInstance();
        self::$_paymentMethod = $paymentMethod;
        self::$_savePaymentMethod = $savePaymentMethod;

        $dateTime = $order['date_added'];

        $paymentMethodId = null;

        // get Data for saved payment method.
        // Saved payment method contains letters and number, but payment method contains just numbers and "-" char
        if (!empty(self::$_paymentMethod) && !ctype_alpha(self::$_paymentMethod) && strpos(self::$_paymentMethod, '-') === false) {

            $customer = Verifone_Storage::getInstance()->getCustomerModel();
            $token = Verifone_Storage::getInstance()->getVerifoneModel()->getCard(self::$_paymentMethod, $customer->isLogged());

            // if for some reason token is wrong, then display all methods
            if ($token == null || empty($token)) {
                self::$_paymentMethod = '';
            } else {
                self::$_paymentMethod = $token['type'];
                $paymentMethodId = $token['token'];
            }

        }

        if (self::$_paymentMethod == Verifone_PaymentMethods::TYPE_ALL) {
            self::$_paymentMethod = '';
        }

        if (self::$_savePaymentMethod == 'on' || self::$_savePaymentMethod == true) {
            self::$_savePaymentMethod = true;
        } else {
            self::$_savePaymentMethod = false;
        }

        $data = [
            'currency_code' => Verifone_System::getCurrencyNumber(),
            'total_incl_amount' => self::_getOrderGrossAmount($order),
            'total_excl_amount' => self::_getOrderNetAmount($order),
            'total_tax' => self::_getOrderTaxAmount($order),
            'order_id' => $order['order_id'],
            'ext_order_id' => '',
            'time' => $dateTime,
            'locale' => Verifone_System::getCustomerLocale(),
            'payment_method' => self::$_paymentMethod,
            'save_payment_method' => self::$_savePaymentMethod
        ];

        if ($paymentMethodId) {
            $data['payment_method_id'] = $paymentMethodId;
        }

        return $data;
    }

    protected static function _calculateTaxPercentage($price, $tax)
    {
        return round($tax * 100 / $price);
    }

    protected static function _getOrderGrossAmount($order)
    {
        return round($order['total'], 2) * 100;
    }

    protected static function _getOrderNetAmount($order)
    {
        return self::_getOrderGrossAmount($order) - self::_getOrderTaxAmount($order);
    }

    protected static function _getOrderTaxAmount($order)
    {
        if(isset($order['totals']['tax'])) {
            return round($order['totals']['tax'], 2) * 100;
        }

        return 0;
    }

    /**
     * @param $order
     * @param $orderItems
     * @param null $paymentMethod
     * @param null $savePaymentMethod
     * @return array
     */
    protected static function _collectProductsData($order, $orderItems, $paymentMethod = null, $savePaymentMethod = null)
    {
        $products = [];

        if (!self::_isSendBasketItems()) {
            return $products;
        }

        self::$_paymentMethod = $paymentMethod;
        self::$_savePaymentMethod = $savePaymentMethod;

        // Group by tax if more products than limit
        if (count($orderItems) >= self::BASKET_LIMIT) {
            $orderItems = self::_groupItemsByTax($orderItems);
        }

        $itemsTax = $itemsNetPrice = $itemsGrossPrice = 0;

        /**
         * @var $orderItem
         */
        foreach ($orderItems as $orderItem) {
            $product = self::_getBasketItemData($orderItem);

            if (!self::_isCombineBasketItems()) {
                $products[] = $product;
            }

            $itemsTax += $product['gross_amount'] - $product['net_amount'];
            $itemsNetPrice += ($product['net_amount']);
            $itemsGrossPrice += ($product['gross_amount']);
        }

        $shippingData = self::_getShippingData($order, $itemsTax);

        if (!is_null($shippingData)) {

            if (!self::_isCombineBasketItems()) {
                $products[] = $shippingData;
            }

            $itemsTax += $shippingData['gross_amount'] - $shippingData['net_amount'];
            $itemsNetPrice += $shippingData['net_amount'];
            $itemsGrossPrice += $shippingData['gross_amount'];

        }

        // add discount product
        $discountAmount = self::_getOrderGrossAmount($order) - $itemsGrossPrice;
        if (abs($discountAmount) >= 1) {
            $discountProduct = self::_getBasketDiscountData(self::_getOrderGrossAmount($order), $itemsGrossPrice,
                self::_getOrderNetAmount($order), $itemsNetPrice);

            $itemsNetPrice += $discountProduct['net_amount'];
            $itemsGrossPrice += $discountProduct['gross_amount'];

            if (!self::_isCombineBasketItems()) {
                $products[] = $discountProduct;
            }
        }

        // combine basket items
        if (self::_isCombineBasketItems()) {
            $products[] = self::_getCombinedBasketItemsData($order, $itemsGrossPrice, $itemsNetPrice, $itemsTax);
        }

        return $products;
    }

    /**
     * @param $orderItem
     *
     * @return array
     */
    protected static function _getBasketItemData($orderItem)
    {
        $itemCount = $orderItem['quantity'];

        $itemNet = $orderItem['price'];
        $itemTax = $orderItem['tax'];

        $itemTaxPercentage = self::_calculateTaxPercentage($itemNet, $itemTax);
        $itemGross = $itemNet + $itemTax;

        return [
            'name' => $orderItem['name'],
            'unit_count' => $itemCount,
            'unit_cost' => round($itemNet, 2) * 100,
            'net_amount' => round($itemNet, 2) * 100 * $itemCount,
            'gross_amount' => round($itemGross, 2) * 100 * $itemCount,
            'tax_percentage' => round($itemTaxPercentage, 2) * 100,
            'discount_percentage' => 0
        ];
    }

    protected static function _getShippingData($order, $itemsTax)
    {
        if (!isset($order['totals']['shipping'])) {
            return null;
        }

        $shippingAmount = (float)$order['totals']['shipping'];
        if ($shippingAmount) {

            $itemNet = round($order['totals']['shipping'], 2);
            if(isset($order['totals']['tax'])) {
                $itemTax = round($order['totals']['tax'] - ($itemsTax / 100), 2);
            } else {
                $itemTax = 0;
            }

            $itemTaxPercentage = round(self::_calculateTaxPercentage($itemNet, $itemTax), 2);
            $itemGross = $itemNet + $itemTax;

            $name = 'Shipping via ' . ucwords($order['shipping_method']);

            return [
                'name' => $name,
                'unit_count' => 1,
                'unit_cost' => $itemNet * 100,
                'net_amount' => $itemNet * 100,
                'gross_amount' => $itemGross * 100,
                'tax_percentage' => $itemTaxPercentage * 100,
                'discount_percentage' => 0
            ];
        }

        return null;
    }

    protected static function _getCombinedBasketItemsData($order, $orderItems, $itemsGrossPrice, $itemsNetPrice, $itemsTax = 0)
    {
        // Must be "Tilaus %ORDERNUMBER%" according to docs
        $itemName = 'Tilaus ' . $order['order_id'];

        $grossAmount = $itemsGrossPrice;
        $netAmount = $itemsNetPrice;

        $count = 0;

        foreach ($orderItems as $orderItem) {
            $count += $orderItem['quantity'];
        }

        return [
            'name' => $itemName,
            'unit_count' => $count,
            'unit_cost' => $netAmount,
            'net_amount' => $netAmount,
            'gross_amount' => $grossAmount,
            'tax_percentage' => round(($itemsTax) / $netAmount, 2) * 100 * 100,
            'discount_percentage' => 0
        ];
    }

    protected static function _groupItemsByTax($items)
    {

        $result = [];

        foreach ($items as $orderItem) {
            $itemNet = round($orderItem['price'], 2);
            $itemTax = round($orderItem['tax'], 2);

            $tax = (string)round(self::_calculateTaxPercentage($itemNet, $itemTax), 2);

            if (!isset($result[$tax])) {

                $mergedItem = [];
                $mergedItem['name'] = sprintf('Multiple items - tax %s', $tax);
                $mergedItem['price'] = (0);
                $mergedItem['tax'] = (0);
                $mergedItem['quantity'] = 1;

                $result[$tax] = $mergedItem;
            }

            $result[$tax]['price'] = ((float)$result[$tax]['price'] + (float)$orderItem['price'] * $orderItem['quantity']);
            $result[$tax]['tax'] = ((float)$result[$tax]['tax'] + (float)$orderItem['tax']);
        }

        return $result;
    }

    protected static function _getBasketDiscountData($orderGrossAmount, $itemsGrossPrice, $orderNetAmount, $itemsNetPrice)
    {

        // calculation for tax return for example 24%, so we have 24/100 => 0.24 but we need int value so *100.
        // But Verifone require round with precision = 2, so again we need *100. 100*100 = 10000
        if (round($orderNetAmount - $itemsNetPrice, 0)) {
            $tax = round(
                    (round($orderGrossAmount - $itemsGrossPrice, 0) - round($orderNetAmount - $itemsNetPrice, 0)
                    ) / round($orderNetAmount - $itemsNetPrice, 0), 2) * 10000;
        } else {
            $tax = 0;
        }

        return [
            'name' => 'Discount',
            'unit_count' => 1,
            'unit_cost' => round($orderNetAmount - $itemsNetPrice, 0),
            'net_amount' => round($orderNetAmount - $itemsNetPrice, 0),
            'gross_amount' => round($orderGrossAmount - $itemsGrossPrice, 0),
            'tax_percentage' => $tax,
            'discount_percentage' => 0
        ];
    }

    protected static function _isSendBasketItems()
    {
        return self::_isSendBasketItemsForAll()
            || (self::_isSendBasketItemsForInvoice() && self::_isMethodTypeInvoice());
    }

    /**
     * Verifone has an option to group basket items into 1 combined item.
     * Currently only possible to enable this for invoice payment methods.
     *
     * @return bool
     */
    protected static function _isCombineBasketItems()
    {
        return (self::_isCombineInvoiceBasketItems() && self::_isMethodTypeInvoice());
    }

    /**
     * If true, basket items are sent for all orders and all payment methods.
     *
     * @return bool
     */
    protected static function _isSendBasketItemsForAll()
    {
        $config = Verifone_Config::getInstance();
        $basketItemSending = $config->getBasketItemSending();;

        return $basketItemSending == Verifone_System::BASKET_ITEMS_SEND_FOR_ALL;
    }

    /**
     * If true, basket items are sent for orders made with invoice payment methods.
     *
     * @return bool
     */
    protected static function _isSendBasketItemsForInvoice()
    {
        $config = Verifone_Config::getInstance();
        $basketItemSending = $config->getBasketItemSending();;

        return $basketItemSending == Verifone_System::BASKET_ITEMS_SEND_FOR_INVOICE;
    }

    protected static function _isCombineInvoiceBasketItems()
    {
        $config = Verifone_Config::getInstance();
        return $config->isCombineInvoiceBasketItems();
    }

    protected static function _isMethodTypeInvoice()
    {

        $paymentMethodCode = self::$_paymentMethod;

        if (!$paymentMethodCode) {
            return false;
        }

        $method = Verifone_PaymentMethods::getPaymentMethodByCode($paymentMethodCode);

        if (is_null($method)) {
            return false;
        }

        return $method['type'] == Verifone_PaymentMethods::TYPE_INVOICE;
    }
}