<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is released under commercial license by Lamia Oy.
 *
 * @copyright Copyright (c) 2017 Lamia Oy (https://lamia.fi)
 * @author    Szymon Nosal <simon@lamia.fi>
 */

use \Verifone\Core\DependencyInjection\CoreResponse\PaymentResponseImpl;
use \Verifone\Core\Converter\Response\CoreResponseConverter;
use \Verifone\Core\DependencyInjection\Service\TransactionImpl;
use \Verifone\Core\DependencyInjection\CoreResponse\PaymentStatusImpl;
use \Verifone\Core\DependencyInjection\CoreResponse\CardImpl;


class Verifone_Payment
{

    const RETRY_DELAY_IN_SECONDS = 2;
    const RETRY_MAX_ATTEMPTS = 5;

    /**
     * @param $order
     * @param PaymentResponseImpl $responseBody
     */
    public static function doneOrder($order, PaymentResponseImpl $responseBody)
    {
        $trans_id = preg_replace("/[^0-9]+/", "", $responseBody->getTransactionNumber());
        $_transactionId = $responseBody->getTransactionNumber();

        // store payment method and transaction id
        $verifoneModel = Verifone_Storage::getInstance()->getVerifoneModel();

        $verifoneOrderId = $verifoneModel->addOrder($order, $_transactionId, $responseBody->getPaymentMethodCode());
        $verifoneModel->addTransaction($verifoneOrderId, 'payment', $responseBody->getOrderGrossAmount() / 100);

        // save masked pan number if available
        self::fetchCards();

        $amount = $responseBody->getOrderGrossAmount() / 100;
        $note = sprintf(Verifone_Storage::getInstance()->getLanguageModel()->get('text_payment_captured'), $_transactionId, $amount);

        self::_addOrderHistory($order['order_id'], $note, true);

    }

    public static function processSuccess($status)
    {

        $params = $_POST;

        $returnData = [
            'status' => $status,
            'error' => true,
            'message' => ''
        ];

        $orderNumber = Verifone_Core_Payment::getOrderNumberFromResponse($params);

        if (empty($orderNumber)) {
            return self::processFail($status);
        }

        $order = self::_getOrder($orderNumber);

        if ($order['order_status_id'] === Verifone_Config::getInstance()->getOrderStatusSuccess()) {
            // Don't need to process paid order.
            // Order changed to paid by delayed functionality
            return self::_returnPaidOrderData($orderNumber, $returnData);
        }

        $attempts = 0;
        while (!Verifone_OrderLocker::lockOrder($orderNumber) && $attempts < self::RETRY_MAX_ATTEMPTS) {
            sleep(self::RETRY_DELAY_IN_SECONDS);
            ++$attempts;
        }

        if ($attempts > 0 && $attempts < self::RETRY_MAX_ATTEMPTS) {
            $orderTmp = self::_getOrder($orderNumber);

            if ($orderTmp['order_status_id'] === Verifone_Config::getInstance()->getOrderStatusSuccess()) {
                // Don't need to process paid order.
                // Order changed to paid by delayed functionality
                return self::_returnPaidOrderData($orderNumber, $returnData);
            }
        }

        try {
            $parsedResponse = Verifone_Core_Payment::validateAndParsePaymentResponse($params, $order);

            /** @var PaymentResponseImpl $body */
            $body = $parsedResponse->getBody();
            $validate = true;
        } catch (Exception $e) {
            return self::processFail($status);
        }

        if ($validate
            && $parsedResponse->getStatusCode() == CoreResponseConverter::STATUS_OK
            && empty($body->getCancelMessage())
        ) {

            self::doneOrder($order, $body);

            Verifone_OrderLocker::unlockOrder($orderNumber);

            $returnData['return_url'] = 'checkout/success';
            $returnData['error'] = false;

        } else {
            return self::processFail($status);
        }

        return $returnData;
    }

    /**
     * @param Order $order
     * @param array $data
     * @return array
     */
    protected static function _returnPaidOrderData($orderNumber, $data)
    {

        $data['return_url'] = 'checkout/success';
        $data['error'] = false;

        if (Verifone_OrderLocker::isLockedOrder($orderNumber)) {
            Verifone_OrderLocker::unlockOrder($orderNumber);
        }

        return $data;
    }


    /**
     * @param $status
     * @return array
     */
    public static function processFail($status)
    {
        $params = $_POST;

        $returnData = [
            'status' => $status,
            'error' => true,
            'message' => ''
        ];

        $orderNumber = Verifone_Core_Payment::getOrderNumberFromResponse($params);

        $returnData['order_id'] = $orderNumber;

        if (empty($orderNumber)) {
            $returnData['return_url'] = 'checkout/checkout';
            return $returnData;
        }

        $order = self::_getOrder($orderNumber);

        try {
            $parsedResponse = Verifone_Core_Payment::validateAndParsePaymentResponse($params, $order);

            /** @var PaymentResponseImpl $body */
            $body = $parsedResponse->getBody();
            $validate = true;
        } catch (Exception $e) {
            $validate = false;
            $parsedResponse = null;
            $body = null;

            $returnData['message'] = Verifone_Storage::getInstance()->getLanguageModel()->get('text_payment_error_security');
        }

        if (!$validate) {
            self::_cancelOrder($order, $returnData['message']);
        } else {
            $returnData['message'] = sprintf(Verifone_Storage::getInstance()->getLanguageModel()->get('text_payment_error_cancel'), $body->getCancelMessage());
            self::_cancelOrder($order, $returnData['message']);
        }

        $returnData['return_url'] = 'checkout/checkout';
        $returnData['order'] = $order;

        return $returnData;
    }

    /**
     * @param $order
     * @param $note
     */
    protected static function _cancelOrder($order, $note)
    {
        self::_addOrderHistory($order['order_id'], $note, false);
    }

    protected static function _confirmPayment($status)
    {
        $confirm = ['committed', 'settled', 'verified'];

        return in_array($status, $confirm);
    }

    protected static function _getOrder($orderId)
    {
        $storage = Verifone_Storage::getInstance();
        $orderModel = $storage->getOrderModel();

        $orderData = $orderModel->getOrder($orderId);

        $orderData['totals'] = [];

        foreach ($orderModel->getOrderTotals($orderId) as $total) {
            $orderData['totals'][$total['code']] = $total['value'];
        }

        return $orderData;
    }

    protected static function _addOrderHistory($orderId, $comment, $success)
    {
        $storage = Verifone_Storage::getInstance();
        $orderModel = $storage->getVerifoneOrderStatus();

        $config = Verifone_Config::getInstance();

        if ($success) {
            $status = $config->getOrderStatusSuccess();
        } else {
            $status = $config->getOrderStatusFail();
        }

        $orderModel->addOrderHistory($orderId, $status, $comment, false, false, true);
    }

    public static function checkPaymentStatus($order, $cron = false)
    {

        if ($cron && !self::transactionsCanBeCheck($order)) {
            return false;
        }

        $response = Verifone_Core_Payment::getTransactionsFromGate($order);

        if (is_null($response)) {
            return false;
        }

        $totalPaid = 0;

        /** @var TransactionImpl $item */
        foreach ($response as $item) {
            $transactionCode = $item->getMethodCode();
            $transactionNumber = $item->getNumber();

            /** @var PaymentStatusImpl $transaction */
            $transaction = Verifone_Core_Payment::getPaymentStatus($transactionCode, $transactionNumber);

            if (!is_null($transaction)) {
                $transactions[] = $transaction;

                $totalPaid += $transaction->getOrderAmount();

                if (self::_confirmPayment($transaction->getCode())) {
                    self::doneOrderBackend($order, $transaction);
                    return true;
                } elseif ($transaction->getCode() == 'cancelled') {
                    return false;
                } elseif ($transaction->getCode() == 'initiated') {
                    return false;
                }
            }

            if ($totalPaid >= $order['total']) {
                return true;
            }

        }

        return null;
    }

    public static function transactionsCanBeCheck($order)
    {
        $orderDate = new \DateTime($order['date_added']);
        $date = new \DateTime();

        $diff = $date->diff($orderDate);

        if ($diff->days > 0 || $diff->h > 1 || $diff->i < 15 || $diff->i > 60) {
            return false;
        }

        if (!self::_isNewLastNote($order)) {
            return false;
        }

        return true;
    }

    protected static function _isNewLastNote($order)
    {

        $note = Verifone_Storage::getInstance()->getVerifoneModel()->getLastOrderHistory($order['order_id']);

        if (empty($note)) {
            return false;
        }

        $noteData = new \DateTime($note['date_added']);
        $date = new \DateTime();

        $diff = $date->diff($noteData);

        if ($diff->days > 0 || $diff->h > 1 || $diff->i < 15) {
            return false;
        }

        return true;
    }

    /**
     * @param $order
     * @param PaymentStatusImpl $responseBody
     */
    public static function doneOrderBackend($order, PaymentStatusImpl $responseBody)
    {
        $trans_id = preg_replace("/[^0-9]+/", "", $responseBody->getTransactionNumber());
        $_transactionId = $responseBody->getTransactionNumber();

        // store payment method and transaction id
        $verifoneModel = Verifone_Storage::getInstance()->getVerifoneModel();

        $verifoneOrderId = $verifoneModel->addOrder($order, $_transactionId, $responseBody->getPaymentMethodCode());
        $verifoneModel->addTransaction($verifoneOrderId, 'payment', $responseBody->getOrderAmount() / 100);

        $amount = $responseBody->getOrderAmount() / 100;
        $note = sprintf(Verifone_Storage::getInstance()->getLanguageModel()->get('text_payment_captured'), $_transactionId, $amount);
        self::_addOrderHistory($order['order_id'], $note, true);

    }

    public static function fetchCards()
    {
        $customer = Verifone_Storage::getInstance()->getCustomerModel();
        if ($customer->isLogged()) {
            $verifoneModel = Verifone_Storage::getInstance()->getVerifoneModel();
            $response =  Verifone_Core_SavedPaymentMethods::getListSavedPaymentMethods();

            if($response === null) {
                return;
            }

            $cards = $response->getBody();
            if (count($cards)) {
                foreach ($cards as $card) {
                    /** @var CardImpl $card */

                    $date = $card->getValidity();
                    if (strlen($date) == 5) {
                        $date = '0' . $date;
                    }
                    $month = substr($date, 0, 2);
                    $year = substr($date, 4, 2);

                    $cardData = [
                        'customer_id' => $customer->isLogged(),
                        'digits' => $card->getTitle(),
                        'expiry' => $month . '/' . $year,
                        'type' => $card->getCode(),
                        'token' => $card->getId()
                    ];

                    $verifoneModel->addCard($cardData);
                }
            }
        }
    }

}
